# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class BGPConfig:
    def __init__(self, data):
        self.id = data.get("id")
        self.status = data.get("status")
        self.deployment_type = data.get("deployment_type")
        self.asn = data.get("asn")
        self.md5 = data.get("md5")
        self.route_object = data.get("route_object")
        self.max_prefix = data.get("max_prefix")
        self.created_at = data.get("created_at")
        self.requested_at = data.get("requested_at")
        self.project = data.get("project")
        self.sessions = data.get("sessions")
        self.ranges = data.get("ranges")
        self.href = data.get("href")

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Organization:
    def __init__(self, data):
        self.id = data.get("id")
        self.name = data.get("name")
        self.description = data.get("description")
        self.website = data.get("website")
        self.twitter = data.get("twitter")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")
        self.tax_id = data.get("tax_id")
        self.main_phone = data.get("main_phone")
        self.billing_phone = data.get("billing_phone")
        self.credit_amount = data.get("credit_amount")
        self.personal = data.get("personal")
        self.customdata = data.get("customdata")
        self.attn = data.get("attn")
        self.purchase_order = data.get("purchase_order")
        self.billing_name = data.get("billing_name")
        self.enforce_2fa = data.get("enforce_2fa")
        self.enforce_2fa_at = data.get("enforce_2fa_at")
        self.short_id = data.get("short_id")
        self.account_id = data.get("account_id")
        self.enabled_features = data.get("enabled_features")
        self.maintenance_email = data.get("maintenance_email")
        self.abuse_email = data.get("abuse_email")
        self.address = data.get("address")
        self.billing_address = data.get("billing_address")
        self.account_manager = data.get("account_manager")
        self.logo = data.get("logo")
        self.logo_thumb = data.get("logo_thumb")
        self.projects = data.get("projects")
        self.plan = data.get("plan")
        self.monthly_spend = data.get("monthly_spend")
        self.current_user_abilities = data.get("current_user_abilities")
        self.href = data.get("href")

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

from .Plan import Plan
from .Project import Project
from .Device import Device


class HardwareReservation:
    def __init__(self, data, manager):
        self.manager = manager

        self.id = data.get("id")
        self.created_at = data.get("created_at")
        self.billing_cycle = data.get("billing_cycle")
        self.created_at = data.get("created_at")
        self.short_id = data.get("short_id")
        self.intervals = data.get("intervals")
        self.current_period = data.get("current_period")
        self.started_at = data.get("started_at")
        self.custom_rate = data.get("custom_rate")
        self.remove_at = data.get("remove_at")
        self.project = data.get("project")
        # self.facility = data.get("facility")
        self.device = data.get("device")
        self.provisionable = data.get("provisionable")
        self.spare = data.get("spare")
        self.need_of_service = data.get("need_of_service")
        self.plan = Plan(data.get("plan"))
        self.switch_uuid = data.get("switch_uuid")

        try:
            project_data = self.manager.call_api(data["project"]["href"], type="GET")
            self.project = Project(project_data, self.manager)
        except (KeyError, IndexError):
            self.attached_to = None

        # endpoint is not working yet
        # try:
        #     facility_data = self.manager.call_api(
        #         data["facility"]["href"], type="GET"
        #     )
        #     self.project = Facility(facility_data, self.manager)
        # except (KeyError, IndexError):
        #     self.attached_to = None

        try:
            device_data = self.manager.call_api(data["device"]["href"], type="GET")
            self.device = Device(device_data, self.manager)
        except (KeyError, IndexError, TypeError):
            self.attached_to = None

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)

    def __getitem__(self, item):
        return getattr(self, item)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Snapshot:
    def __init__(self, data):
        self.id = data.get("id")
        self.status = data.get("status")
        self.timestamp = data.get("timestamp")
        self.created_at = data.get("created_at")
        self.volume = data.get("volume")

    def __str__(self):
        return "%s" % self.name

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Project:
    def __init__(self, data, manager):
        self.manager = manager

        self.id = data.get("id")
        self.name = data.get("name")
        self.payment_method = data.get("payment_method", [])
        self.max_projects = data.get("max_devices")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")
        self.devices = data.get("devices")
        self.invitations = data.get("invitations")
        self.memberships = data.get("memberships")
        self.members = data.get("members")
        self.ssh_keys = data.get("ssh_keys")

    def update(self):
        params = {"name": self.name}

        return self.manager.call_api(
            "projects/%s" % self.id, type="PATCH", params=params
        )

    def delete(self):
        return self.manager.call_api("projects/%s" % self.id, type="DELETE")

    def __str__(self):
        return "%s" % self.name

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import unittest
import packet
import random


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestEmail(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])

        cls.email = cls.manager.add_email(
            "john.doe{}@packet.com".format(random.randint(1, 1001))
        )

    def test_get_email(self):
        email = self.manager.get_email(self.email.id)
        self.assertEqual(email.address, self.email.address)

    def test_update_email(self):
        self.email.address = "john.doe{}@packet.com".format(random.randint(1, 1001))
        self.email.update()
        # email address cannot be updated?
        # email = self.manager.get_email(self.email.id)
        # self.assertEqual(email.address, self.email.address)

    @classmethod
    def tearDownClass(cls):
        cls.email.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Provider:
    def __init__(self, data):
        self.id = data.get("id")
        self.name = data.get("name")
        self.status = data.get("status")
        self.type = data.get("type")
        self.public = data.get("public")

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Event:
    def __init__(self, data):
        self.id = data.get("id")
        self.type = data.get("type")
        self.body = data.get("body")
        self.state = data.get("state")
        self.created_at = data.get("created_at")
        self.modified_by = data.get("modified_by")
        self.ip = data.get("ip")
        self.interpolated = data.get("interpolated")

    def __str__(self):
        return "%s" % self.interpolated

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

from __future__ import print_function

import os
import sys
import unittest
import packet


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestVpn(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])
        self.manager.turn_on_vpn()

    # def test_get_vpn_config(self):
    #     config = self.manager.get_vpn_configuration("ewr1")
    #     print(config)

    @classmethod
    def tearDownClass(self):
        self.manager.turn_off_vpn()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import time
import unittest
import packet

from datetime import datetime


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestPorts(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])

        org_id = self.manager.list_organizations()[0].id
        self.project = self.manager.create_organization_project(
            org_id=org_id,
            name="Int-Tests-Ports_{}".format(
                datetime.utcnow().strftime("%Y%m%dT%H%M%S.%f")[:-3]
            ),
        )

        self.device = self.manager.create_device(
            self.project.id, "networktestingdevice", "baremetal_2", "ewr1", "centos_7"
        )
        self.vlan = self.manager.create_vlan(self.project.id, "ewr1")
        self.vlan2 = self.manager.create_vlan(self.project.id, "ewr1")

        while True:
            if self.manager.get_device(self.device.id).state == "active":
                break
            time.sleep(2)
        self.device_port_id = self.device["network_ports"][0]["id"]
        self.device_eth0_port_id = self.device["network_ports"][1]["id"]

    def test01_convert_layer2(self):
        self.manager.convert_layer_2(self.device_port_id, self.vlan.id)

    def test02_remove_port(self):
        self.manager.remove_port(self.device_port_id, self.vlan.id)

    def test03_assign_port(self):
        self.manager.assign_port(self.device_port_id, self.vlan.id)

    def test04_bond_port(self):
        self.manager.bond_ports(self.device_port_id, False)

    def test05_disbond_port(self):
        self.manager.disbond_ports(self.device_port_id, False)

    def test06_assign_native_vlan(self):
        # must remove vlan from any previous association and attach more than one vlan to the eth0 port to be able to
        #  choose a native vlan
        self.manager.remove_port(self.device_port_id, self.vlan.id)
        self.manager.assign_port(self.device_eth0_port_id, self.vlan.id)
        self.manager.assign_port(self.device_eth0_port_id, self.vlan2.id)
        self.manager.assign_native_vlan(self.device_eth0_port_id, self.vlan.id)

    def test07_remove_native_vlan(self):
        self.manager.remove_native_vlan(self.device_eth0_port_id)

    def test08_convert_layer3(self):
        ipadresses = list({"address_family": 6, "public": False})
        self.manager.convert_layer_3(self.device_port_id, ipadresses)

    @classmethod
    def tearDownClass(self):
        self.manager.remove_port(self.device_eth0_port_id, self.vlan.id)
        self.device.delete()
        self.vlan2.delete()
        self.vlan.delete()
        self.project.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Facility:
    def __init__(self, data):
        self.id = data.get("id")
        self.code = data.get("code")
        self.name = data.get("name")
        self.features = data.get("features")
        self.address = data.get("address")

    def __str__(self):
        return "%s" % self.code

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import unittest
import packet

from datetime import datetime


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestEvent(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])

        cls.events = cls.manager.list_events()

        org_id = cls.manager.list_organizations()[0].id
        cls.project = cls.manager.create_organization_project(
            org_id=org_id,
            name="Int-Tests-Events_{}".format(
                datetime.utcnow().strftime("%Y%m%dT%H%M%S.%f")[:-3]
            ),
        )

    def test_list_events(self):
        self.assertTrue(len(self.events) > 0)

    def test_get_event(self):
        event = self.manager.get_event(self.events[0].id)
        self.assertEqual(event.id, self.events[0].id)

    def test_get_project_events(self):
        events = self.manager.list_project_events(self.project.id)
        self.assertGreaterEqual(len(events), 0)

    @classmethod
    def tearDownClass(cls):
        cls.project.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class SSHKey:
    def __init__(self, data, manager):
        self.manager = manager

        self.id = data.get("id")
        self.key = data.get("key")
        self.label = data.get("label")
        self.fingerprint = data.get("fingerprint")
        self.href = data.get("href")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")

        try:
            self.owner = data["owner"]["href"]
        except (KeyError, TypeError):
            self.owner = None

    def update(self):
        params = {"label": self.label, "key": self.key}

        return self.manager.call_api(
            "ssh-keys/%s" % self.id, type="PATCH", params=params
        )

    def delete(self):
        return self.manager.call_api("ssh-keys/%s" % self.id, type="DELETE")

    def __str__(self):
        return "%s %s" % (self.id, self.label)

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only
# from .Volume import Volume
from .OperatingSystem import OperatingSystem


class Device:
    def __init__(self, data, manager):
        self.manager = manager

        self.id = data.get("id")
        self.short_id = data.get("short_id")
        self.hostname = data.get("hostname")
        self.description = data.get("description")
        self.state = data.get("state")
        self.tags = data.get("tags")
        self.image_url = data.get("image_url")
        self.billing_cycle = data.get("billing_cycle")
        self.user = data.get("user")
        self.iqn = data.get("iqn")
        self.locked = data.get("locked")
        self.bonding_mode = data.get("bonding_mode")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")
        self.ipxe_script_url = data.get("ipxe_script_url")
        self.always_pxe = data.get("always_pxe", False)
        self.storage = data.get("storage")
        self.customdata = data.get("customdata")
        self.operating_system = OperatingSystem(data["operating_system"])
        self.facility = data.get("facility")
        self.metro = data.get("metro")
        self.project = data.get("project")
        self.ssh_keys = data.get("ssh_keys")
        self.project_lite = data.get("project_lite")
        self.volumes = data.get("volumes")
        self.ip_addresses = data.get("ip_addresses")
        self.plan = data.get("plan")
        self.userdata = data.get("userdata")
        self.switch_uuid = data.get("switch_uuid")
        self.network_ports = data.get("network_ports")
        self.href = data.get("href")
        self.spot_instance = data.get("spot_instance", False)
        self.hardware_reservation_id = data.get("hardware_reservation_id")
        self.spot_price_max = data.get("spot_price_max")
        self.termination_time = data.get("termination_time")
        self.root_password = data.get("root_password")
        self.provisioning_percentage = data.get("provisioning_percentage")

    def update(self):
        params = {
            "hostname": self.hostname,
            "locked": self.locked,
            "tags": self.tags,
            "description": self.description,
            "billing_cycle": self.billing_cycle,
            "userdata": self.userdata,
            "always_pxe": self.always_pxe,
            "ipxe_script_url": self.ipxe_script_url,
            "spot_instance": self.spot_instance,
            "customdata": self.customdata,
        }

        return self.manager.call_api(
            "devices/%s" % self.id, type="PATCH", params=params
        )

    def delete(self):
        return self.manager.call_api("devices/%s" % self.id, type="DELETE")

    def reinstall(self, operating_system=None, ipxe_script_url=None):
        params = {"type": "reinstall"}
        if operating_system is not None:
            params["operating_system"] = operating_system
        if ipxe_script_url is not None:
            params["operating_system"] = "custom_ipxe"
            params["ipxe_script_url"] = ipxe_script_url

        return self.manager.call_api(
            "devices/%s/actions" % self.id, type="POST", params=params
        )

    def power_off(self):
        params = {"type": "power_off"}
        return self.manager.call_api(
            "devices/%s/actions" % self.id, type="POST", params=params
        )

    def power_on(self):
        params = {"type": "power_on"}
        return self.manager.call_api(
            "devices/%s/actions" % self.id, type="POST", params=params
        )

    def reboot(self):
        params = {"type": "reboot"}
        return self.manager.call_api(
            "devices/%s/actions" % self.id, type="POST", params=params
        )

    def rescue(self):
        params = {"type": "rescue"}
        return self.manager.call_api(
            "devices/%s/actions" % self.id, type="POST", params=params
        )

    def ips(self):
        return self.manager.list_device_ips(self.id)

    def __str__(self):
        return "%s" % self.hostname

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)

    def __getitem__(self, item):
        return getattr(self, item)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import unittest
import time
import packet

from datetime import datetime


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestVolume(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.timestamp = ""
        self.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])
        self.projectId = self.manager.list_projects()[0].id

        org_id = self.manager.list_organizations()[0].id
        self.project = self.manager.create_organization_project(
            org_id=org_id,
            name="Int-Tests-Volume_{}".format(
                datetime.utcnow().strftime("%Y%m%dT%H%M%S.%f")[:-3]
            ),
        )

        self.volume = self.manager.create_volume(
            self.project.id, "volume description", "storage_1", "100", "ewr1", 7, "1day"
        )

        while True:
            if self.manager.get_volume(self.volume.id).state == "active":
                break
            time.sleep(2)

        self.device = self.manager.create_device(
            self.project.id, "devicevolumestest", "baremetal_0", "ewr1", "centos_7"
        )

        self.policy = self.volume.create_snapshot_policy("1day", 2)
        self.clone = self.volume.clone()

        while True:
            if self.manager.get_device(self.device.id).state == "active":
                break
            time.sleep(2)

    def test_get_volume(self):
        volume = self.manager.get_volume(self.volume.id)
        self.assertEqual(volume.description, self.volume.description)

    def test_list_volumes(self):
        volumes = self.manager.list_volumes(self.project.id)
        for volume in volumes:
            if volume.id is self.volume.id:
                break
        self.assertRaises(TypeError)

    def test_update_volume(self):
        self.volume.description = "newdescription"
        self.volume.update()
        volume = self.manager.get_volume(self.volume.id)
        self.assertEqual(self.volume.description, volume.description)

    def test_attach_volume(self):
        self.volume.attach(self.device.id)
        while True:
            if self.manager.get_device(self.device.id).state == "active":
                break
            time.sleep(1)

    def test_detach_volume(self):
        self.volume.detach()
        while True:
            if self.manager.get_device(self.device.id).state == "active":
                break
            time.sleep(1)

    def test_create_snapshot(self):
        self.volume.create_snapshot()

    def test_get_snapshots(self):
        snapshots = self.manager.get_snapshots(self.volume.id)
        self.assertIsNotNone(snapshots)
        self.__class__.timestamp = snapshots[0].timestamp

    def test_update_snapshot_policy(self):
        self.policy = self.policy.update_snapshot_policy("1month", 3)
        assert self.policy.frequency == "1month"
        assert self.policy.count == 3

    def test_restore_volume(self):
        self.volume.restore(self.__class__.timestamp)

    @classmethod
    def tearDownClass(self):
        self.policy.delete()
        self.volume.delete()
        self.device.delete()
        self.clone.delete()
        self.project.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class OperatingSystem(object):
    def __init__(self, data):
        self.slug = data.get("slug")
        self.name = data.get("name")
        self.distro = data.get("distro")
        self.version = data.get("version")
        self.provisionable_on = data.get("provisionable_on")

    def __str__(self):
        return "%s %s %s %s" % (self.slug, self.name, self.distro, self.version)

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.slug)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class BGPSession:
    def __init__(self, data):
        self.id = data.get("id")
        self.status = data.get("status")
        self.learned_routes = data.get("learned_routes")
        self.switch_name = data.get("switch_name")
        self.default_route = data.get("default_route")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")
        self.device = data.get("device")
        self.address_family = data.get("address_family")
        self.href = data.get("href")

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

"""library to interact with the Equinix Metal API"""

__version__ = "1.44.3"
__author__ = "Equinix Metal Engineers"
__author_email__ = "support@equinixmetal.com"
__license__ = "LGPL v3"
__copyright__ = "Copyright (c) 2021, Equinix"

from .Device import Device  # noqa
from .Email import Email  # noqa
from .Event import Event  # noqa
from .Facility import Facility  # noqa
from .Metro import Metro  # noqa
from .OperatingSystem import OperatingSystem  # noqa
from .Plan import Plan  # noqa
from .Project import Project  # noqa
from .SSHKey import SSHKey  # noqa
from .Volume import Volume  # noqa
from .BGPConfig import BGPConfig  # noqa
from .BGPSession import BGPSession  # noqa
from .DeviceBatch import DeviceBatch  # noqa
from .Manager import Manager  # noqa
from .Snapshot import Snapshot  # noqa
from .Organization import Organization  # noqa
from .Provider import Provider  # noqa
from .baseapi import Error  # noqa
from .baseapi import ResponseError  # noqa



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Batch:
    def __init__(self, data):
        self.id = data.get("id")
        self.error_messages = data.get("error_messages")
        self.quantity = data.get("quantity")
        self.state = data.get("state")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")
        self.devices = data.get("devices")
        self.project = data.get("project")
        self.state = data.get("state")
        self.error_messages = data.get("error_messages")

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only
from packet.Vlan import Vlan
from .baseapi import BaseAPI
from .baseapi import Error as PacketError
from .baseapi import ResponseError
from .Batch import Batch
from .Plan import Plan
from .Device import Device
from .SSHKey import SSHKey
from .Project import Project
from .Facility import Facility
from .Metro import Metro
from .OperatingSystem import OperatingSystem
from .Volume import Volume
from .BGPConfig import BGPConfig
from .BGPSession import BGPSession
from .IPAddress import IPAddress
from .HardwareReservation import HardwareReservation
from .Snapshot import Snapshot
from .Organization import Organization
from .Email import Email
from .Event import Event
from .Provider import Provider


class Manager(BaseAPI):
    def __init__(self, auth_token, consumer_token=None):
        super(Manager, self).__init__(auth_token, consumer_token)

    def call_api(self, method, type="GET", params=None):
        return super(Manager, self).call_api(method, type, params)

    def get_user(self):
        return self.call_api("user")

    def list_facilities(self, params={}):
        data = self.call_api("facilities", params=params)
        facilities = list()
        for jsoned in data["facilities"]:
            facility = Facility(jsoned)
            facilities.append(facility)
        return facilities

    def list_metros(self, params={}):
        data = self.call_api("locations/metros", params=params)
        metros = list()
        for jsoned in data["metros"]:
            metro = Metro(jsoned)
            metros.append(metro)
        return metros

    def list_plans(self, params={}):
        data = self.call_api("plans", params=params)
        plans = list()
        for jsoned in data["plans"]:
            plan = Plan(jsoned)
            plans.append(plan)
        return plans

    def list_operating_systems(self, params={}):
        data = self.call_api("operating-systems", params=params)
        oss = list()
        for jsoned in data["operating_systems"]:
            os = OperatingSystem(jsoned)
            oss.append(os)
        return oss

    def list_projects(self, params={}):
        data = self.call_api("projects", params=params)
        self.total = data["meta"]["total"]
        projects = list()
        for jsoned in data["projects"]:
            project = Project(jsoned, self)
            projects.append(project)
        return projects

    def get_project(self, project_id):
        data = self.call_api("projects/%s" % project_id)
        return Project(data, self)

    def create_project(self, name):
        params = {"name": name}
        data = self.call_api("projects", type="POST", params=params)
        return Project(data, self)

    def list_hardware_reservations(self, project_id, params={}):
        data = self.call_api(
            "projects/%s/hardware-reservations" % project_id, params=params
        )
        hardware_reservations = list()
        for jsoned in data["hardware_reservations"]:
            hardware_reservation = HardwareReservation(jsoned, self)
            hardware_reservations.append(hardware_reservation)
        return hardware_reservations

    def get_hardware_reservation(self, hardware_reservation_id):
        data = self.call_api("hardware-reservations/%s" % hardware_reservation_id)
        return HardwareReservation(data, self)

    def list_devices(self, project_id, params={}):
        data = self.call_api("projects/%s/devices" % project_id, params=params)
        devices = list()
        for jsoned in data["devices"]:
            device = Device(jsoned, self)
            devices.append(device)
        return devices

    def list_all_devices(self, project_id):
        raw_devices = list()
        page = 1
        while True:
            paginate = {"page": page}
            data = self.call_api("projects/%s/devices" % project_id, params=paginate)
            next = self.meta["next"]
            raw_devices.extend(data["devices"])
            if next is None:
                break
            else:
                page += 1

        all_devices = list()
        for raw_device in raw_devices:
            device = Device(raw_device, self)
            all_devices.append(device)
        return all_devices

    def create_device(
        self,
        project_id,
        hostname,
        plan,
        facility="",
        operating_system="",
        always_pxe=False,
        billing_cycle="hourly",
        features={},
        ipxe_script_url="",
        locked=False,
        project_ssh_keys=[],
        public_ipv4_subnet_size=31,
        spot_instance=False,
        spot_price_max=-1,
        tags={},
        termination_time=None,
        user_ssh_keys=[],
        userdata="",
        hardware_reservation_id="",
        storage={},
        customdata={},
        metro="",
    ):

        params = {
            "billing_cycle": billing_cycle,
            "features": features,
            "hostname": hostname,
            "locked": locked,
            "operating_system": operating_system,
            "plan": plan,
            "project_id": project_id,
            "public_ipv4_subnet_size": public_ipv4_subnet_size,
            "project_ssh_keys": project_ssh_keys,
            "tags": tags,
            "user_ssh_keys": user_ssh_keys,
            "userdata": userdata,
        }

        if metro != "":
            params["metro"] = metro
        if facility != "":
            params["facility"] = facility
        if hardware_reservation_id != "":
            params["hardware_reservation_id"] = hardware_reservation_id
        if storage:
            params["storage"] = storage
        if customdata:
            params["customdata"] = customdata
        if ipxe_script_url != "":
            params["always_pxe"] = always_pxe
            params["ipxe_script_url"] = ipxe_script_url
            params["operating_system"] = "custom_ipxe"
        if spot_instance:
            params["spot_instance"] = spot_instance
            params["spot_price_max"] = spot_price_max
            params["termination_time"] = termination_time
        data = self.call_api(
            "projects/%s/devices" % project_id, type="POST", params=params
        )
        return Device(data, self)

    def get_device(self, device_id):
        data = self.call_api("devices/%s" % device_id)
        return Device(data, self)

    def list_ssh_keys(self, params={}):
        data = self.call_api("ssh-keys", params=params)
        ssh_keys = list()
        for jsoned in data["ssh_keys"]:
            ssh_key = SSHKey(jsoned, self)
            ssh_keys.append(ssh_key)
        return ssh_keys

    def get_ssh_key(self, ssh_key_id):
        data = self.call_api("ssh-keys/%s" % ssh_key_id)
        return SSHKey(data, self)

    def create_ssh_key(self, label, public_key):
        params = {"key": public_key, "label": label}
        data = self.call_api("ssh-keys", type="POST", params=params)
        return SSHKey(data, self)

    def create_project_ssh_key(self, project_id, label, public_key):
        """
        Successfully creating an SSH key with a Project API Token results
        in a 404 from the API. If we get a 404, we try the request again.

        If the request actually failed with a 404, we will get another 404
        which we raise.

        If the request actually succeeded, we will get a 422. In this case,
        we will try to list all the keys and find the SSHKey we just
        received.

        Customer Report Reference: TUVD-0107-UIKB
        """

        def issue_req():
            try:
                params = {"key": public_key, "label": label}
                data = self.call_api(
                    "projects/%s/ssh-keys" % project_id, type="POST", params=params
                )
                return SSHKey(data, self)
            except ResponseError as e:
                if e.response.status_code == 422:
                    # Try to pluck the SSH key from the listing API
                    keys = [
                        key
                        for key in self.list_ssh_keys()
                        if key.key.strip() == public_key.strip()
                    ]
                    if len(keys) == 1:
                        return keys.pop()
                raise

        try:
            return issue_req()
        except ResponseError as e:
            if e.response.status_code == 404:
                return issue_req()
            else:
                raise

    def list_volumes(self, project_id, params={}):
        params["include"] = "facility,attachments.device"
        data = self.call_api("projects/%s/storage" % project_id, params=params)
        volumes = list()
        for jsoned in data["volumes"]:
            volume = Volume(jsoned, self)
            volumes.append(volume)
        return volumes

    def create_volume(
        self,
        project_id,
        description,
        plan,
        size,
        facility,
        snapshot_count=0,
        snapshot_frequency=None,
    ):
        params = {
            "description": description,
            "plan": plan,
            "size": size,
            "facility": facility,
        }

        if snapshot_count > 0 and snapshot_frequency is not None:
            params["snapshot_policies"] = {
                "snapshot_count": snapshot_count,
                "snapshot_frequency": snapshot_frequency,
            }

        data = self.call_api(
            "projects/%s/storage?include=facility" % project_id,
            type="POST",
            params=params,
        )

        return Volume(data, self)

    def get_volume(self, volume_id):
        params = {"include": "facility,attachments.device"}
        data = self.call_api("storage/%s" % volume_id, params=params)
        return Volume(data, self)

    def get_capacity(self, legacy=None):
        """Get capacity of all facilities.

        :param legacy: Indicate set of server types to include in response

        Validation of `legacy` is left to the packet api to avoid going out
        of date if any new value is introduced.
        The currently known values are:
          - only (current default, will be switched "soon")
          - include
          - exclude (soon to be default)
        """
        params = None
        if legacy:
            params = {"legacy": legacy}

        return self.call_api("/capacity", params=params)["capacity"]

    # servers is a list of tuples of facility, plan, and quantity.
    def validate_capacity(self, servers):
        params = {"servers": []}
        for server in servers:
            params["servers"].append(
                {"facility": server[0], "plan": server[1], "quantity": server[2]}
            )

        try:
            data = self.call_api("/capacity", "POST", params)
            return all(s["available"] for s in data["servers"])
        except PacketError as e:  # pragma: no cover
            if e.args[0] == "Error 503: Service Unavailable":
                return False
            else:
                raise e

    # servers is a list of tuples of metro, plan, and quantity.
    def validate_metro_capacity(self, servers):
        params = {"servers": []}
        for server in servers:
            params["servers"].append(
                {"metro": server[0], "plan": server[1], "quantity": server[2]}
            )

        try:
            data = self.call_api("/capacity/metros", "POST", params)
            return all(s["available"] for s in data["servers"])
        except PacketError as e:  # pragma: no cover
            if e.args[0] == "Error 503: Service Unavailable":
                return False
            else:
                raise e

    def get_spot_market_prices(self, params={}):
        data = self.call_api("/market/spot/prices", params=params)
        return data["spot_market_prices"]

    # BGP Config
    def get_bgp_config(self, project_id):
        data = self.call_api("projects/%s/bgp-config" % project_id)
        return BGPConfig(data)

    def enable_project_bgp_config(
        self, project_id, asn, deployment_type, md5=None, use_case=None
    ):
        params = {
            "deployment_type": deployment_type,
            "asn": asn,
            "md5": md5,
            "use_case": use_case,
        }
        self.call_api(
            "/projects/%s/bgp-configs" % project_id, type="POST", params=params
        )

    # BGP Session
    def get_bgp_sessions(self, device_id, params={}):
        data = self.call_api(
            "/devices/%s/bgp/sessions" % device_id, type="GET", params=params
        )
        bgp_sessions = list()
        for jsoned in data["bgp_sessions"]:
            bpg_session = BGPSession(jsoned)
            bgp_sessions.append(bpg_session)
        return bgp_sessions

    def create_bgp_session(self, device_id, address_family):
        data = self.call_api(
            "/devices/%s/bgp/sessions" % device_id,
            type="POST",
            params={"address_family": address_family},
        )
        return BGPSession(data)

    # IP operations
    def list_device_ips(self, device_id):
        data = self.call_api("devices/%s/ips" % device_id, type="GET")
        ips = list()
        for jsoned in data["ip_addresses"]:
            ip = IPAddress(jsoned, self)
            ips.append(ip)
        return ips

    def get_ip(self, ip_id):
        data = self.call_api("ips/%s" % ip_id)
        return IPAddress(data, self)

    def delete_ip(self, ip_id):
        self.call_api("ips/%s" % ip_id, type="DELETE")

    def list_project_ips(self, project_id, params={}):
        data = self.call_api("projects/%s/ips" % project_id, type="GET", params=params)
        ips = list()
        for jsoned in data["ip_addresses"]:
            ip = IPAddress(jsoned, self)
            ips.append(ip)
        return ips

    def get_available_ip_subnets(self, ip_id, cidr):
        data = self.call_api(
            "/ips/%s/available" % ip_id, type="GET", params="cidr=%s" % cidr
        )
        return data

    def create_device_ip(self, device_id, address, manageable=True, customdata=None):
        params = {
            "address": address,
            "manageable": manageable,
            "customdata": customdata,
        }

        data = self.call_api("/devices/%s/ips" % device_id, params=params, type="POST")
        return IPAddress(data, self)

    def reserve_ip_address(
        self,
        project_id,
        type,
        quantity,
        facility="",
        details=None,
        comments=None,
        tags=list(),
        metro="",
    ):
        request = {
            "type": type,
            "quantity": quantity,
            "details": details,
            "comments": comments,
            "tags": tags,
        }

        if facility != "":
            request["facility"] = facility
        if metro != "":
            request["metro"] = metro
        data = self.call_api(
            "/projects/%s/ips" % project_id, params=request, type="POST"
        )
        return IPAddress(data, self)

    # Batches
    def create_batch(self, project_id, params):
        param = {"batches": params}
        data = self.call_api(
            "/projects/%s/devices/batch" % project_id, type="POST", params=param
        )
        batches = list()
        for b in data["batches"]:
            batch = Batch(b)
            batches.append(batch)
        return batches

    def list_batches(self, project_id, params=None):
        data = self.call_api(
            "/projects/%s/batches" % project_id, type="GET", params=params
        )
        batches = list()
        for b in data["batches"]:
            batch = Batch(b)
            batches.append(batch)
        return batches

    def delete_batch(self, batch_id, remove_associated_instances=False):
        self.call_api(
            "/batches/%s" % batch_id, type="DELETE", params=remove_associated_instances
        )

    # Snapshots
    def get_snapshots(self, volume_id, params=None):
        data = self.call_api(
            "storage/%s/snapshots" % volume_id, type="GET", params=params
        )
        snapshots = list()
        for ss in data["snapshots"]:
            snapshot = Snapshot(ss)
            snapshots.append(snapshot)

        return snapshots

    def restore_volume(self, volume_id, restore_point):
        params = {"restore_point": restore_point}
        self.call_api("storage/%s/restore" % volume_id, type="POST", params=params)

    # Organization
    def list_organizations(self, params=None):
        data = self.call_api("organizations", type="GET", params=params)
        orgs = list()
        for org in data["organizations"]:
            o = Organization(org)
            orgs.append(o)

        return orgs

    def get_organization(self, org_id, params=None):
        data = self.call_api("organizations/%s" % org_id, type="GET", params=params)
        return Organization(data)

    def list_organization_projects(self, org_id, params=None):
        data = self.call_api(
            "organizations/%s/projects" % org_id, type="GET", params=params
        )
        projects = list()
        for p in data["projects"]:
            projects.append(Project(p, self))

        return projects

    def list_organization_devices(self, org_id, params=None):
        data = self.call_api(
            "organizations/%s/devices" % org_id, type="GET", params=params
        )
        devices = list()
        for d in data["devices"]:
            devices.append(Device(d, self))

        return devices

    def create_organization_project(
        self, org_id, name, payment_method_id=None, customdata=None
    ):
        params = {
            "name": name,
            "payment_method_id": payment_method_id,
            "customdata": customdata,
        }
        data = self.call_api(
            "organizations/%s/projects" % org_id, type="POST", params=params
        )
        return Project(data, self)

    # Email
    def add_email(self, address, default=False):
        params = {"address": address, "default": default}
        data = self.call_api("emails", type="POST", params=params)
        return Email(data, self)

    def get_email(self, email_id):
        data = self.call_api("emails/%s" % email_id)
        return Email(data, self)

    # Event
    def list_events(self, params=None):
        data = self.call_api("events", type="GET", params=params)
        events = list()
        for e in data["events"]:
            events.append(Event(e))

        return events

    def get_event(self, event_id):
        data = self.call_api("events/%s" % event_id)
        return Event(data)

    def list_device_events(self, device_id, params=None):
        data = self.call_api("devices/%s/events" % device_id, type="GET", params=params)
        events = list()
        for e in data["events"]:
            events.append(Event(e))

        return events

    def list_project_events(self, project_id, params=None):
        data = self.call_api(
            "projects/%s/events" % project_id, type="GET", params=params
        )
        events = list()
        for e in data["events"]:
            events.append(Event(e))

        return events

    def get_volume_events(self, volume_id, params=None):
        data = self.call_api("volumes/%s/events" % volume_id, type="GET", params=params)
        events = list()
        for e in data["events"]:
            events.append(Event(e))

        return events

    # vlan operations
    def list_vlans(self, project_id, params=None):
        data = self.call_api(
            "projects/%s/virtual-networks" % project_id, type="GET", params=params
        )
        vlans = list()
        for vlan in data["virtual_networks"]:
            vlans.append(Vlan(vlan, self))

        return vlans

    def create_vlan(
        self, project_id, facility="", vxlan=None, vlan=None, description=None, metro=""
    ):
        params = {
            "project_id": project_id,
            "vxlan": vxlan,
            "vlan": vlan,
            "description": description,
        }
        if facility != "":
            params["facility"] = facility
        if metro != "":
            params["metro"] = metro
        data = self.call_api(
            "projects/%s/virtual-networks" % project_id, type="POST", params=params
        )
        return Vlan(data, self)

    def assign_port(self, port_id, vlan_id):
        params = {"vnid": vlan_id}
        self.call_api("ports/%s/assign" % port_id, type="POST", params=params)

    def remove_port(self, port_id, vlan_id):
        params = {"vnid": vlan_id}
        self.call_api("ports/%s/unassign" % port_id, type="POST", params=params)

    def disbond_ports(self, port_id, bulk_disable):
        params = {"bulk_disable": bulk_disable}
        self.call_api("ports/%s/disbond" % port_id, type="POST", params=params)

    def bond_ports(self, port_id, bulk_disable):
        params = {"bulk_disable": bulk_disable}
        self.call_api("ports/%s/bond" % port_id, type="POST", params=params)

    def convert_layer_2(self, port_id, vlan_id):
        params = {"vnid": vlan_id}
        self.call_api("ports/%s/convert/layer-2" % port_id, type="POST", params=params)

    def convert_layer_3(self, port_id, request_ips):
        params = {"request_ips": request_ips}
        self.call_api("ports/%s/convert/layer-3" % port_id, type="POST", params=params)

    def assign_native_vlan(self, port_id, vnid):
        params = {"vnid": vnid}
        self.call_api("ports/%s/native-vlan" % port_id, type="POST", params=params)

    def remove_native_vlan(self, port_id):
        self.call_api("ports/%s/native-vlan" % port_id, type="DELETE")

    def get_vpn_configuration(self, facilityCode):
        params = {"code": facilityCode}
        data = self.call_api("user/vpn", type="GET", params=params)
        return data

    def turn_on_vpn(self):
        return self.call_api("user/vpn", type="POST")

    def turn_off_vpn(self):
        return self.call_api("user/vpn", type="DELETE")

    # Packet connect
    def create_packet_connections(self, params):
        body = {
            "name": params["name"],
            "project_id": params["project_id"],
            "provider_id": params["provider_id"],
            "provider_payload": params["provider_payload"],
            "port_speed": params["port_speed"],
            "vlan": params["vlan"],
        }
        for opt in ["tags", "description", "facility", "metro"]:
            if opt in params:
                body[opt] = params[opt]

        data = self.call_api("/packet-connect/connections", type="POST", params=body)
        return data

    def get_packet_connection(self, connection_id):
        data = self.call_api("/packet-connect/connections/%s" % connection_id)
        return data

    def delete_packet_connection(self, connection_id):
        data = self.call_api(
            "/packet-connect/connections/%s" % connection_id, type="DELETE"
        )
        return data

    def provision_packet_connection(self, connection_id):
        data = self.call_api(
            "/packet-connect/connections/{id}/provision" % connection_id, type="POST"
        )
        return data

    def deprovision_packet_connection(self, connection_id, delete):
        params = {"delete": delete}
        data = self.call_api(
            "/packet-connect/connections/{id}/deprovision" % connection_id,
            type="POST",
            params=params,
        )
        return data

    def list_packet_connect_providers(self):
        data = self.call_api("/packet-connect/providers", type="GET")
        providers = list()
        for p in data["providers"]:
            providers.append(Provider(p))
        return providers

    def get_packet_connect_provider(self, provider_id):
        data = self.call_api("/packet-connect/providers/%s" % provider_id, type="GET")
        return Provider(data)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import unittest
import time
import packet

from datetime import datetime


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestDevice(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])
        org_id = self.manager.list_organizations()[0].id
        self.project = self.manager.create_organization_project(
            org_id=org_id,
            name="Int-Tests-Device_{}".format(
                datetime.utcnow().strftime("%Y%m%dT%H%M%S.%f")[:-3]
            ),
        )

        self.manager.enable_project_bgp_config(
            project_id=self.project.id, deployment_type="local", asn=65000
        )

        self.device = self.manager.create_device(
            self.project.id, "devicetest", "baremetal_0", "ewr1", "centos_7"
        )

        while True:
            if self.manager.get_device(self.device.id).state == "active":
                break
            time.sleep(2)

    def test_get_device(self):
        device = self.manager.get_device(self.device.id)
        self.assertEqual(device.hostname, self.device.hostname)

    def test_list_devices(self):
        devices = self.manager.list_devices(self.project.id)
        for device in devices:
            if device.id is self.device.id:
                break
        self.assertRaises(TypeError)

    def test_update_device(self):
        self.device.hostname = "newname"
        self.device.update()
        device = self.manager.get_device(self.device.id)
        self.assertEqual(self.device.hostname, device.hostname)

    def test_create_bgp_session(self):
        bgp_session = self.manager.create_bgp_session(
            self.device.id, address_family="ipv4"
        )
        self.assertIsNotNone(bgp_session)

    def test_get_bgp_sessions(self):
        data = self.manager.get_bgp_sessions(self.device.id)
        self.assertIsNotNone(self, data)

    def test_get_device_events(self):
        events = self.manager.list_device_events(self.device.id)
        self.assertGreater(len(events), 0)

    def test_get_device_ips(self):
        ips = self.device.ips()
        self.assertTrue(len(ips) > 0)

    @classmethod
    def tearDownClass(self):
        self.device.delete()
        self.project.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import time
import unittest
import packet

from datetime import datetime


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestIps(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])

        org_id = cls.manager.list_organizations()[0].id
        cls.project = cls.manager.create_organization_project(
            org_id=org_id,
            name="Int-Tests-IPs_{}".format(
                datetime.utcnow().strftime("%Y%m%dT%H%M%S.%f")[:-3]
            ),
        )

        cls.ip_block = cls.manager.reserve_ip_address(
            project_id=cls.project.id,
            type="public_ipv4",
            quantity=1,
            facility="ewr1",
            details="delete me",
            tags=["deleteme"],
        )

        cls.device = cls.manager.create_device(
            cls.project.id, "iptest", "baremetal_0", "ewr1", "centos_7"
        )

        while True:
            if cls.manager.get_device(cls.device.id).state == "active":
                break
            time.sleep(2)

    def test_reserve_ip_address(self):
        self.assertEqual(32, self.ip_block.cidr)
        self.assertEqual("delete me", self.ip_block.details)

    def test_list_project_ips(self):
        ips = self.manager.list_project_ips(self.project.id)
        self.assertGreater(len(ips), 0)

    def test_create_device_ip(self):
        ip = self.manager.create_device_ip(
            self.device.id, address=self.ip_block.address
        )
        self.assertIsNotNone(ip)
        self.assertEqual(ip.address, self.ip_block.address)

    @classmethod
    def tearDownClass(cls):
        cls.device.delete()
        cls.ip_block.delete()
        cls.project.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import sys
import json
import unittest

import packet
import requests_mock


class PacketManagerTest(unittest.TestCase):
    def setUp(self):
        self.manager = PacketMockManager(auth_token="foo")

    def test_get_user(self):
        user = self.manager.get_user()

        self.assertEqual(user.get("full_name"), "Aaron Welch")

    def test_list_facilities(self):
        facilities = self.manager.list_facilities()
        for facility in facilities:
            str(facility)
            repr(facility)
            self.assertIsInstance(facility, packet.Facility)

    def test_list_metros(self):
        metros = self.manager.list_metros()
        for metro in metros:
            str(metro)
            repr(metro)
            self.assertIsInstance(metro, packet.Metro)

    def test_list_plans(self):
        plans = self.manager.list_plans()
        for plan in plans:
            str(plan)
            repr(plan)
            self.assertIsInstance(plan, packet.Plan)

    def test_list_operating_systems(self):
        oss = self.manager.list_operating_systems()
        for o in oss:
            str(o)
            repr(o)
            self.assertIsInstance(o, packet.OperatingSystem)

    def test_list_projects(self):
        projects = self.manager.list_projects()
        self.assertIsInstance(projects, list)
        for project in projects:
            str(project)
            repr(project)
            self.assertIsInstance(project, packet.Project)

    def test_get_project(self):
        project = self.manager.get_project("438659f0")
        self.assertIsInstance(project, packet.Project)

    def test_create_project(self):
        project = self.manager.create_project("test project")
        self.assertIsInstance(project, packet.Project)

    def test_update_project(self):
        name = "updated name"
        project = self.manager.get_project("438659f0")
        project.name = name
        project.update()
        self.assertEqual(project.name, name)
        self.assertIsInstance(project, packet.Project)

    def test_delete_project(self):
        project = self.manager.get_project("438659f0")
        self.assertIsNone(project.delete())

    def test_list_devices(self):
        devices = self.manager.list_devices("438659f0")
        for device in devices:
            str(device)
            repr(device)
            self.assertIsInstance(device, packet.Device)

    # TODO figure out how to properly handle this test case
    # def test_list_all_devices(self):
    #     devices = self.manager.list_all_devices("438659f0")
    #     for device in devices:
    #         str(device)
    #         repr(device)
    #         self.assertIsInstance(device, packet.Device)

    def test_create_device(self):
        device = self.manager.create_device(
            "438659f0", "hostname", "baremetal_0", "ewr1", "ubuntu_14_04"
        )
        self.assertIsInstance(device, packet.Device)

    def test_create_device_ipxe(self):
        device = self.manager.create_device(
            "438659f0",
            "hostname",
            "baremetal_0",
            "ewr1",
            "custom_ipxe",
            ipxe_script_url="https://example.com",
            always_pxe=True,
        )
        self.assertIsInstance(device, packet.Device)

    def test_get_device(self):
        device = self.manager.get_device("9dec7266")
        self.assertIsInstance(device, packet.Device)

    def test_device_actions(self):
        device = self.manager.get_device("9dec7266")
        self.assertIsNone(device.power_off())
        self.assertIsNone(device.power_on())
        self.assertIsNone(device.rescue())
        self.assertIsNone(device.reboot())

    def test_update_device(self):
        hostname = "updated hostname"
        device = self.manager.get_device("9dec7266")
        device.hostname = hostname
        device.update()
        self.assertEqual(device.hostname, hostname)
        self.assertIsInstance(device, packet.Device)

    def test_delete_device(self):
        device = self.manager.get_device("9dec7266")
        self.assertIsNone(device.delete())

    def test_list_ssh_keys(self):
        keys = self.manager.list_ssh_keys()
        for key in keys:
            str(key)
            repr(key)
            self.assertIsInstance(key, packet.SSHKey)

    def test_get_ssh_key(self):
        key = self.manager.get_ssh_key("084a5dec")
        self.assertIsInstance(key, packet.SSHKey)

    def test_create_ssh_key(self):
        public_key = "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDI4pIqzpb5g3992h+yr527VRcaB68KE4vPjWPPoiQws49KIs2NMcOzS9QE4641uW1u5ML2HgQdfYKMF/YFGnI1Y6xV637DjhDyZYV9LasUH49npSSJjsBcsk9JGfUpNAOdcgpFzK8V90eiOrOC5YncxdwwG8pwjFI9nNVPCl4hYEu1iXdyysHvkFfS2fklsNjLWrzfafPlaen+qcBxygCA0sFdW/7er50aJeghdBHnE2WhIKLUkJxnKadznfAge7oEe+3LLAPfP+3yHyvp2+H0IzmVfYvAjnzliYetqQ8pg5ZW2BiJzvqz5PebGS70y/ySCNW1qQmJURK/Wc1bt9en"

        key = self.manager.create_ssh_key(label="sshkey-name", public_key=public_key)
        self.assertIsInstance(key, packet.SSHKey)
        self.assertEqual(key.key, public_key)

    def test_delete_ssh_key(self):
        key = self.manager.get_ssh_key("084a5dec")
        self.assertIsNone(key.delete())

    def test_update_ssh_key(self):
        label = "updated label"
        key = self.manager.get_ssh_key("084a5dec")
        key.label = label
        key.update()
        self.assertEqual(key.label, label)
        self.assertIsInstance(key, packet.SSHKey)

    def test_list_volumes(self):
        volumes = self.manager.list_volumes("438659f0")
        for volume in volumes:
            self.assertIsInstance(volume, packet.Volume)

    def test_create_volume(self):
        volume = self.manager.create_volume(
            "438659f0", "volume description", "storage_0", "100", "ewr1", 7, "1day"
        )
        self.assertIsInstance(volume, packet.Volume)

    def test_get_volume(self):
        volume = self.manager.get_volume("f9a8a263")
        str(volume)
        repr(volume)
        self.assertIsInstance(volume, packet.Volume)

    def test_update_volume(self):
        description = "updated description"
        volume = self.manager.get_volume("f9a8a263")
        volume.description = description
        volume.update()
        self.assertEqual(volume.description, description)
        self.assertIsInstance(volume, packet.Volume)

    def test_delete_volume(self):
        volume = self.manager.get_volume("f9a8a263")
        self.assertIsNone(volume.delete())

    def test_list_volume_snapshots(self):
        volume = self.manager.get_volume("f9a8a263")
        snaps = volume.list_snapshots()
        for snap in snaps:
            str(snap)
            repr(snap)
            snap.delete()

    def test_attach_volume(self):
        volume = self.manager.get_volume("f9a8a263")
        self.assertIsNone(volume.attach("9dec7266"))

    def test_detach_volume(self):
        volume = self.manager.get_volume("f9a8a263")
        self.assertIsNone(volume.detach())

    def test_volume_create_snapshot(self):
        volume = self.manager.get_volume("f9a8a263")
        volume.create_snapshot()

    def test_volume_create_clone(self):
        volume = self.manager.get_volume("f9a8a263")
        volume.clone()

    def test_capacity(self):
        self.manager.get_capacity()

    def test_get_bgp_config(self):
        bgp = self.manager.get_bgp_config("1234")
        self.assertIsNotNone(bgp)

    def test_validate_capacity(self):
        capacity = self.manager.validate_capacity([("ewr1", "baremetal_0", 10)])
        self.assertTrue(capacity)

    def test_validate_metro_capacity(self):
        capacity = self.manager.validate_metro_capacity([("sv", "baremetal_1", 10)])
        self.assertTrue(capacity)

    # IP Addresses
    def test_list_device_ips(self):
        ips = self.manager.list_device_ips("e123s")
        self.assertIsNotNone(ips)

    def test_list_projects_ips(self):
        ips = self.manager.list_project_ips("438659f0")
        self.assertIsNotNone(ips)
        for ip in ips:
            self.assertIsInstance(ip.facility, packet.Facility)

    def test_list_projects_ips_state_all(self):
        ips = self.manager.list_project_ips("438659f1", params={"state": "all"})
        self.assertIsNotNone(ips)
        self.assertIsNone(ips[0].facility)
        self.assertIsInstance(ips[1].facility, packet.Facility)


class PacketMockManager(packet.Manager):
    def call_api(self, method, type="GET", params=None):
        with requests_mock.Mocker() as m:
            mock = {
                "DELETE": m.delete,
                "GET": m.get,
                "PUT": m.put,
                "POST": m.post,
                "PATCH": m.patch,
            }[type]

            if type == "DELETE":
                mock(requests_mock.ANY)
                return super(PacketMockManager, self).call_api(method, type, params)

            fixture = "%s_%s" % (type.lower(), method.lower())
            fixture = fixture.replace("/", "_").split("?")[0]
            fixture = "test/fixtures/%s.json" % fixture

            headers = {"content-type": "application/json"}

            with open(fixture) as data_file:
                j = json.load(data_file)

            mock(requests_mock.ANY, headers=headers, json=j)
            return super(PacketMockManager, self).call_api(method, type, params)


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only
from packet import Project
from .Facility import Facility
from .Metro import Metro


class Vlan:
    def __init__(self, data, manager):
        self.manager = manager
        if data is None:
            return

        self.id = data.get("id")
        self.description = data.get("description")
        self.vxlan = data.get("vxlan")
        self.internet_gateway = data.get("internet_gateway")
        self.facility_code = data.get("facility_code")
        self.metro_code = data.get("metro_code")
        self.created_at = data.get("created_at")
        facility = data.get("facility")
        self.facility = Facility(facility) if facility else None
        metro = data.get("metro")
        self.metro = Metro(metro) if metro else None

        try:
            project_data = self.manager.call_api(
                data["assigned_to"]["href"], type="GET"
            )
            self.assigned_to = Project(project_data, self.manager)
        except (KeyError, IndexError):
            self.attached_to = None

    def get(self):
        return self.manager.call_api("virtual-networks/%s" % self.id, type="GET")

    def delete(self):
        return self.manager.call_api("virtual-networks/%s" % self.id, type="DELETE")

    def create_internet_gateway(self, ip_reservation_length):
        """:param ip_reservation_length:  (required) number of IP addresses possible 8 or 16"""
        params = {"length": ip_reservation_length}
        return self.manager.call_api(
            "/virtual-networks/%s/internet-gateways" % self.id,
            type="POST",
            params=params,
        )

    def assign_native_vlan(self, port_id):
        params = {"vnid": self.id}
        return self.manager.call_api(
            "/ports/%s/native-vlan" % port_id, type="POST", params=params
        )

    def remove_native_vlan(self, port_id):
        return self.manager.call_api("/ports/%s/native-vlan" % port_id, type="DELETE")

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Plan:
    def __init__(self, data):
        self.id = data.get("id")
        self.name = data.get("name")
        self.slug = data.get("slug")
        self.line = data.get("line")
        self.pricing = data.get("pricing")
        self.specs = data.get("specs")
        self.description = data.get("description")
        self.available_in = data.get("available_in")

    def __str__(self):
        return "%s" % self.slug

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

from .Facility import Facility
from .Metro import Metro


class IPAddress:
    def __init__(self, data, manager):
        self.manager = manager

        self.id = data.get("id")
        self.address_family = data.get("address_family")
        self.netmask = data.get("netmask")
        self.created_at = data.get("created_at")
        self.details = data.get("details")
        self.tags = data.get("tags")
        self.public = data.get("public")
        self.cidr = data.get("cidr")
        self.management = data.get("management")
        self.enabled = data.get("enabled")
        self.global_ip = data.get("global_ip")
        self.customdata = data.get("customdata")
        self.project = data.get("project")
        self.project_lite = data.get("project_lite")
        self.details = data.get("details")
        self.assigned_to = data.get("assigned_to")
        self.interface = data.get("interface")
        self.network = data.get("network")
        self.address = data.get("address")
        self.gateway = data.get("gateway")

        facility = data.get("facility")
        self.facility = Facility(facility) if facility else None
        metro = data.get("metro")
        self.metro = Metro(metro) if metro else None

    def delete(self):
        return self.manager.call_api("ips/%s" % self.id, type="DELETE")

    def __str__(self):
        return "%s" % self.code

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import unittest
import packet


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestOrganization(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])
        orgs = self.manager.list_organizations()
        self.org_id = orgs[0].id

    def test_organization(self):
        org = self.manager.get_organization(org_id=self.org_id)
        self.assertEqual(self.org_id, org.id)

    def test_create_organization_project(self):
        project = self.manager.create_organization_project(
            org_id=self.org_id,
            name="live-tests-project",
            payment_method_id=None,
            customdata={"tag": "delete me"},
        )
        self.assertIsNotNone(project)
        project.delete()

    def test_list_organization_projects(self):
        projects = self.manager.list_organization_projects(org_id=self.org_id)
        self.assertGreater(len(projects), 0)

    def test_list_organization_devices(self):
        devices = self.manager.list_organization_devices(org_id=self.org_id)
        self.assertGreaterEqual(len(devices), 0)

    @classmethod
    def tearDownClass(self):
        pass


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

from .Plan import Plan
from .Facility import Facility


class Volume:
    def __init__(self, data, manager):
        self.manager = manager
        if data is None:
            return

        self.id = data.get("id")
        self.name = data.get("name")
        self.description = data.get("description")
        self.size = data.get("size")
        self.state = data.get("state")
        self.locked = data.get("locked")
        self.billing_cycle = data.get("billing_cycle")
        self.created_at = data.get("created_at")
        self.updated_at = data.get("updated_at")
        self.attachments = data.get("attachments")

        self.plan = Plan(data.get("plan"))
        self.facility = Facility(data.get("facility"))
        try:
            self.attached_to = data["attachments"][0]["device"]["id"]
        except (KeyError, IndexError):
            self.attached_to = None

    def update(self):
        params = {
            "description": self.description,
            "size": self.size,
            "plan": self.plan.slug,
            "locked": self.locked,
        }

        return self.manager.call_api(
            "storage/%s" % self.id, type="PATCH", params=params
        )

    def delete(self):
        return self.manager.call_api("storage/%s" % self.id, type="DELETE")

    def attach(self, device_id):
        params = {"device_id": device_id}
        return self.manager.call_api(
            "storage/%s/attachments" % self.id, type="POST", params=params
        )

    def detach(self):
        for attachment in self.attachments:
            return self.manager.call_api(attachment["href"], type="DELETE")

    def list_snapshots(self, params={}):
        data = self.manager.call_api("storage/%s/snapshots" % (self.id))
        snapshots = list()
        for jsoned in data["snapshots"]:
            snapshot = VolumeSnapshot(jsoned, self)
            snapshots.append(snapshot)
        return snapshots

    def create_snapshot(self):
        self.manager.call_api("storage/%s/snapshots" % self.id, type="POST")

    def create_snapshot_policy(self, frequency, count):
        """Creates a new snapshot policy of your volume.

        :param frequency:  (required) Snapshot frequency

        Validation of `frequency` is left to the packet api to avoid going out
        of date if any new value is introduced.
        The currently known values are:
          - 1hour,
          - 1day
          - 1week
          - 1month
          - 1year
        """
        data = self.manager.call_api(
            "storage/{0}/snapshot-policies?snapshot_frequency={1}&snapshot_count={2}".format(
                self.id, frequency, count
            ),
            type="POST",
        )
        return SnapshotPolicy(data, self)

    def clone(self):
        return Volume(
            self.manager.call_api("storage/%s/clone" % self.id, type="POST"),
            manager=self.manager,
        )

    def restore(self, restore_point):
        self.manager.restore_volume(self.id, restore_point=restore_point)

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)


class VolumeSnapshot:
    def __init__(self, data, volume):
        self.volume = volume

        self.id = data["id"]
        self.status = data["status"]
        self.timestamp = data["timestamp"]
        self.created_at = data["created_at"]

    def delete(self):
        return self.volume.manager.call_api(
            "/storage/%s/snapshots/%s" % (self.volume.id, self.id), type="DELETE"
        )

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)


class SnapshotPolicy:
    def __init__(self, data, policy):
        self.policy = policy

        self.id = data["id"]
        self.count = data["snapshot_count"]
        self.frequency = data["snapshot_frequency"]
        self.created_at = data["created_at"]
        self.updated_at = data["updated_at"]

    def delete(self):
        return self.policy.manager.call_api(
            "storage/snapshot-policies/%s" % self.id, type="DELETE"
        )

    def update_snapshot_policy(self, frequency, count):
        """Updates the volume snapshot policy.

        :param frequency:  (required) Snapshot frequency

        Validation of `frequency` is left to the packet api to avoid going out
        of date if any new value is introduced.
        The currently known values are:
          - 1hour,
          - 1day
          - 1week
          - 1month
          - 1year
        """
        data = self.policy.manager.call_api(
            "storage/snapshot-policies/{0}?snapshot_frequency={1}&snapshot_count={2}".format(
                self.id, frequency, count
            ),
            type="PATCH",
        )
        return SnapshotPolicy(data, self)

    def __str__(self):
        return "%s" % self.id

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import sys
import unittest

import packet


class obj(object):
    def __init__(self, dict_):
        self.__dict__.update(dict_)


class ErrorTest(unittest.TestCase):
    def test_cause(self):
        msg = "boom"
        cause = "cause"
        error = packet.Error(msg, cause)
        self.assertIn(error.cause, cause)


class BaseAPITest(unittest.TestCase):
    def setUp(self):
        self.auth_token = "fake_auth_token"
        self.consumer_token = "fake_consumer_token"
        self.end_point = "api.packet.net"
        self._user_agent_prefix = "fake_user_agent"

    def test_init_all(self):
        base = packet.baseapi.BaseAPI(
            self.auth_token, self.consumer_token, self._user_agent_prefix
        )
        self.assertEqual(base.end_point, self.end_point)
        self.assertEqual(base.auth_token, self.auth_token)
        self.assertEqual(base.consumer_token, self.consumer_token)
        self.assertEqual(base._user_agent_prefix, self._user_agent_prefix)

    def test_call_api_with_end_point(self):
        base = packet.baseapi.BaseAPI(
            self.auth_token, self.consumer_token, self._user_agent_prefix
        )

        if int(sys.version[0]) == 3:
            self.assertRaisesRegex(
                packet.Error,
                "method type not recognized as one of",
                base.call_api,
                "fake_path",
                "bad_method",
            )


class ResponseErrorTest(unittest.TestCase):
    def setUp(self):
        self.resp500 = obj({"status_code": 500})
        self.errBoom = {"error": "boom"}
        self.errBangBoom = {"errors": ["bang", "boom"]}
        self.exception = Exception("x")

    def test_init_empty(self):
        error = packet.ResponseError(self.resp500, None, None)
        self.assertIn("empty", str(error))

    def test_init_string(self):
        error = packet.ResponseError(self.resp500, "whoops", None)
        self.assertIn("whoops", str(error))

    def test_init_error(self):
        error = packet.ResponseError(self.resp500, self.errBoom, self.exception)
        self.assertIn("Error 500: boom", str(error))
        self.assertEqual(500, error.response.status_code)
        self.assertEqual(self.exception, error.cause)

    def test_init_errors(self):
        error = packet.ResponseError(self.resp500, self.errBangBoom, self.exception)
        self.assertIn("Error 500: bang, boom", str(error))
        self.assertEqual(500, error.response.status_code)
        self.assertEqual(self.exception, error.cause)


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import json
import logging
import requests

from packet import __version__


class Error(Exception):
    """Base exception class for this module"""

    def __init__(self, msg, cause=None):
        super(Error, self).__init__(msg)
        self._cause = cause

    @property
    def cause(self):
        """The underlying exception causing the error, if any."""
        return self._cause


class ResponseError(Error):
    def __init__(self, resp, data, exception=None):
        if not data:
            msg = "(empty response)"
        elif not isinstance(data, dict):
            msg = str(data)
        elif "error" in data:
            msg = data["error"]
        elif "errors" in data:
            msg = ", ".join(data["errors"])
        super(ResponseError, self).__init__(
            "Error {0}: {1}".format(resp.status_code, msg), exception
        )
        self._response = resp

    @property
    def response(self):
        """The Requests response which failed"""
        return self._response


class JSONReadError(Error):
    pass


class BaseAPI(object):
    """
    Basic api class for
    """

    def __init__(self, auth_token, consumer_token, user_agent=""):
        self.auth_token = auth_token
        self.consumer_token = consumer_token
        self.end_point = "api.packet.net"
        self._user_agent_prefix = user_agent
        self._log = logging.getLogger(__name__)

    @property
    def user_agent(self):
        return "{}packet-python/{} {}".format(
            self._user_agent_prefix, __version__, requests.utils.default_user_agent()
        ).strip()

    def call_api(self, method, type="GET", params=None):  # noqa
        if params is None:
            params = {}

        url = "https://" + self.end_point + "/" + method

        headers = {
            "X-Auth-Token": self.auth_token,
            "X-Consumer-Token": self.consumer_token,
            "Content-Type": "application/json",
            "User-Agent": self.user_agent,
        }

        # remove token from log
        headers_str = str(headers).replace(self.auth_token.strip(), "TOKEN")
        self._log.debug("%s %s %s %s" % (type, url, params, headers_str))
        try:
            if type == "GET":
                url = url + "%s" % self._parse_params(params)
                resp = requests.get(url, headers=headers)
            elif type == "POST":
                resp = requests.post(
                    url,
                    headers=headers,
                    data=json.dumps(
                        params, default=lambda o: o.__dict__, sort_keys=True, indent=4
                    ),
                )
            elif type == "DELETE":
                resp = requests.delete(url, headers=headers)
            elif type == "PATCH":
                resp = requests.patch(url, headers=headers, data=json.dumps(params))
            else:
                raise Error(
                    "method type not recognized as one of GET, POST, DELETE or PATCH: %s"
                    % type
                )
        except requests.exceptions.RequestException as e:  # pragma: no cover
            raise Error("Communcations error: %s" % str(e), e)

        if not resp.content:
            data = None
        elif resp.headers.get("content-type", "").startswith("application/json"):
            try:
                data = resp.json()
            except ValueError as e:  # pragma: no cover
                raise JSONReadError("Read failed: %s" % e.message, e)
        else:
            data = resp.content  # pragma: no cover

        if not resp.ok:  # pragma: no cover
            raise ResponseError(resp, data)

        try:
            resp.raise_for_status()
        except requests.HTTPError as e:  # pragma: no cover
            raise ResponseError(resp, data, e)

        self.meta = None
        try:
            if data and data["meta"]:
                self.meta = data["meta"]
        except (KeyError, IndexError):
            pass

        return data

    def _parse_params(self, params):
        vals = list()
        for k, v in params.items():
            vals.append(str("%s=%s" % (k, v)))
        return "?" + "&".join(vals)



#!/usr/bin/env python
# SPDX-License-Identifier: LGPL-3.0-only
import codecs
import os.path

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools

    use_setuptools()
    from setuptools import setup


with open("README.md") as readme, open("CHANGELOG.md") as changelog:
    long_description = readme.read() + "\n" + changelog.read()


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


setup(
    name="packet-python",
    version=get_version("packet/__init__.py"),
    description="Equinix Metal (Packet) API client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/packethost/packet-python",
    author="Equinix Metal Developers",
    author_email="support@equinixmetal.com",
    license="LGPL v3",
    keywords="equinix metal packet api client infrastructure",
    packages=["packet"],
    install_requires="requests",
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "requests-mock"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class Email:
    def __init__(self, data, manager):
        self.manager = manager

        self.id = data.get("id")
        self.address = data.get("address")
        self.default = data.get("default")
        self.verified = data.get("verified")

    def update(self):
        params = {"address": self.address, "default": self.default}

        return self.manager.call_api("emails/%s" % self.id, type="PATCH", params=params)

    def delete(self):
        return self.manager.call_api("emails/%s" % self.id, type="DELETE")

    def __str__(self):
        return "%s" % self.address

    def __repr__(self):
        return "{}: {}".format(self.__class__.__name__, self.id)



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only

import os
import sys
import time
import unittest
import packet

from datetime import datetime


@unittest.skipIf(
    "PACKET_PYTHON_TEST_ACTUAL_API" not in os.environ,
    "PACKET_PYTHON_TEST_ACTUAL_API is missing from environment",
)
class TestBatches(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.manager = packet.Manager(auth_token=os.environ["PACKET_AUTH_TOKEN"])
        org_id = self.manager.list_organizations()[0].id
        self.project = self.manager.create_organization_project(
            org_id=org_id,
            name="Int-Tests-Batch_{}".format(
                datetime.utcnow().strftime("%Y%m%dT%H%M%S.%f")[:-3]
            ),
        )
        self.batches = list()

    def test_create_batch(self):
        params = list()
        batch01 = packet.DeviceBatch(
            {
                "hostname": "batchtest01",
                "quantity": 1,
                "facility": "ewr1",
                "operating_system": "centos_7",
                "plan": "baremetal_0",
            }
        )

        params.append(batch01)
        data = self.manager.create_batch(project_id=self.project.id, params=params)
        self.batches = data
        time.sleep(10)

    def test_list_batches(self):
        self.manager.list_batches(project_id=self.project.id)

    def test_delete_batches(self):
        self.batches = self.manager.list_batches(project_id=self.project.id)
        for batch in self.batches:
            self.manager.delete_batch(batch.id, remove_associated_instances=True)

    @classmethod
    def tearDownClass(self):
        devices = self.manager.list_devices(project_id=self.project.id)
        for device in devices:
            if device.hostname == "batchtest01":
                if device.state != "active":
                    while True:
                        if self.manager.get_device(device.id).state != "active":
                            time.sleep(2)
                        else:
                            device.delete()
                            break
        self.project.delete()


if __name__ == "__main__":
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
# SPDX-License-Identifier: LGPL-3.0-only


class DeviceBatch:
    def __init__(self, data):
        self.hostname = data.get("hostname")
        self.plan = data.get("plan")
        self.operating_system = data.get("operating_system")
        self.facility = data.get("facility")
        self.metro = data.get("metro")
        self.quantity = data.get("quantity")

    def __str__(self):
        return "%s" % self.hostname
